"""Added image_size attribute to PackageExtractRun

Revision ID: dd19dd789f6d
Revises: a05ad6e1d0ca
Create Date: 2019-10-29 09:07:00.504320+00:00

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = 'dd19dd789f6d'
down_revision = 'a05ad6e1d0ca'
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column('package_extract_run', sa.Column('image_size', sa.Integer(), nullable=True))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('package_extract_run', 'image_size')
    # ### end Alembic commands ###
